/*-------------------------------------------------------------------
   ALLCOLOR.C -- Palette Animation Using Windows 3.0 Palette Manager
                 (c) Charles Petzold, 1990
  -------------------------------------------------------------------*/

#include <windows.h>

long FAR PASCAL WndProc (HWND, WORD, WORD, LONG) ;

#define ID_TIMER    1

int PASCAL WinMain (HANDLE hInstance, HANDLE hPrevInstance,
                    LPSTR lpszCmdLine, int nCmdShow)
     {
     char     szAppName [] = "AllColor" ;
     HWND     hwnd ;
     MSG      msg ;
     WNDCLASS wndclass ;

     if (!hPrevInstance) 
          {
          wndclass.style         = CS_HREDRAW | CS_VREDRAW ;
          wndclass.lpfnWndProc   = WndProc ;
          wndclass.cbClsExtra    = 0 ;
          wndclass.cbWndExtra    = 0 ;
          wndclass.hInstance     = hInstance ;
          wndclass.hIcon         = NULL ;
          wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
          wndclass.hbrBackground = GetStockObject (WHITE_BRUSH) ;
          wndclass.lpszMenuName  = szAppName ;
          wndclass.lpszClassName = szAppName ;

          RegisterClass (&wndclass) ;
          }

     hwnd = CreateWindow (szAppName, "All Colors: Palette Animation",
                          WS_OVERLAPPEDWINDOW,
                          CW_USEDEFAULT, CW_USEDEFAULT,
                          CW_USEDEFAULT, CW_USEDEFAULT,
                          NULL, NULL, hInstance, NULL) ;

     ShowWindow (hwnd, nCmdShow) ;
     UpdateWindow (hwnd) ;

     while (GetMessage (&msg, NULL, 0, 0))
          {
          TranslateMessage (&msg) ;
          DispatchMessage (&msg) ;
          }
     return msg.wParam ;
     }

void DisplayRGB (HDC hdc, PALETTEENTRY *ppe)
     {
     char szBuffer [11] ;

     SetTextColor (hdc, RGB (0, 0, 0)) ;
     SetBkColor (hdc, RGB (255, 255, 255)) ;
     SetBkMode (hdc, OPAQUE) ;

     wsprintf (szBuffer, " %02X %02X %02X ",
               ppe->peRed, ppe->peGreen, ppe->peBlue) ;

     TextOut (hdc, 0, 0, szBuffer, 10) ;
     }

long FAR PASCAL WndProc (HWND hwnd, WORD message, WORD wParam, LONG lParam)
     {
     static BOOL         bBlueUp = TRUE, bGreenUp = TRUE, bRedUp = TRUE ;
     static HPALETTE     hPal ;
     static int          iIncr ;
     static PALETTEENTRY pe ;
     BOOL                bPalSupport ;
     HBRUSH              hBrush ;
     HDC                 hdc ;
     LOGPALETTE          lp ;
     PAINTSTRUCT         ps ;
     RECT                rc ;

     switch (message)
          {
          case WM_CREATE:

                    // Check for palette support

               hdc = GetDC (hwnd) ;

               bPalSupport = (RC_PALETTE & GetDeviceCaps (hdc, RASTERCAPS)) &&
                             (GetDeviceCaps (hdc, SIZEPALETTE) >
                              GetDeviceCaps (hdc, NUMRESERVED)) ;

                    // Determine the color resolution and set iIncr ;

               iIncr = 1 << (8 - GetDeviceCaps (hdc, COLORRES) / 3) ;

               ReleaseDC (hwnd, hdc) ;

               if (!bPalSupport)
                    {
                    MessageBox (hwnd, "Program requires palette support",
                                "AllClr", MB_ICONEXCLAMATION | MB_OK) ;

                    DestroyWindow (hwnd) ;
                    return 0 ;
                    }

                    // Initialize the fields of the LOGPALETTE structure

               lp.palVersion             = 0x300 ;
               lp.palNumEntries          = 1 ;
               lp.palPalEntry[0].peRed   = 0 ;
               lp.palPalEntry[0].peGreen = 0 ;
               lp.palPalEntry[0].peBlue  = 0 ;
               lp.palPalEntry[0].peFlags = PC_RESERVED ;

                    // Create the logical palette

               hPal = CreatePalette (&lp) ;

                    // Initialize the PALETTEENTRY structure & set the timer

               pe = lp.palPalEntry[0] ;

               SetTimer (hwnd, ID_TIMER, 50, NULL) ;
               return 0 ;

          case WM_TIMER:

                    // Define new color value

               pe.peBlue += (bBlueUp ? iIncr : -iIncr) ;

               if (pe.peBlue == (bBlueUp ? 0 : 256 - iIncr))
                    {
                    pe.peBlue = (bBlueUp ? 256 - iIncr : 0) ;
                    bBlueUp ^= TRUE ;
                    pe.peGreen += (bGreenUp ? iIncr : -iIncr) ;

                    if (pe.peGreen == (bGreenUp ? 0 : 256 - iIncr))
                         {
                         pe.peGreen = (bGreenUp ? 256 - iIncr : 0) ;
                         bGreenUp ^= TRUE ;
                         pe.peRed += (bRedUp ? iIncr : -iIncr) ;

                         if (pe.peRed == (bRedUp ? 0 : 256 - iIncr))
                              {
                              pe.peRed = (bRedUp ? 256 - iIncr : 0) ;
                              bRedUp ^= TRUE ;
                              }
                         }
                    }

                    // Animate the palette

               AnimatePalette (hPal, 0, 1, &pe) ;

               hdc = GetDC (hwnd) ;
               DisplayRGB (hdc, &pe) ;
               ReleaseDC (hwnd, hdc) ;

               return 0 ;

          case WM_PAINT:
               hdc = BeginPaint (hwnd, &ps) ;

               SelectPalette (hdc, hPal, FALSE) ;
               RealizePalette (hdc) ;

               hBrush = CreateSolidBrush (PALETTEINDEX (0)) ;

               GetClientRect (hwnd, &rc) ;
               FillRect (hdc, &rc, hBrush) ;

               DisplayRGB (hdc, lp.palPalEntry) ;

               EndPaint (hwnd, &ps) ;
               DeleteObject (hBrush) ;
               return 0 ;

          case WM_QUERYNEWPALETTE:
               hdc = GetDC (hwnd) ;

               SelectPalette (hdc, hPal, FALSE) ;

               if (RealizePalette (hdc) > 0)
                    {
                    ReleaseDC (hwnd, hdc) ;
                    InvalidateRect (hwnd, NULL, FALSE) ;
                    return TRUE ;
                    }
               else
                    {
                    ReleaseDC (hwnd, hdc) ;
                    return FALSE ;
                    }
               break ;

          case WM_PALETTECHANGED:
               if (wParam != hwnd)
                    {
                    hdc = GetDC (hwnd) ;

                    SelectPalette (hdc, hPal, FALSE) ;

                    if (RealizePalette (hdc) > 0)
                         {
                         InvalidateRect (hwnd, NULL, FALSE) ;
                         }

                    ReleaseDC (hwnd, hdc) ;
                    }
               return 0 ;

          case WM_DESTROY :

                    // Kill the timer

               KillTimer (hwnd, ID_TIMER) ;

                    // Delete the logical palette

               DeleteObject (hPal) ;
               PostQuitMessage (0) ;
               return 0 ;
          }
     return DefWindowProc (hwnd, message, wParam, lParam) ;
     }
