/*----------------------------------------------------------------
   FADER.C -- Palette Animation Using Windows 3.0 Palette Manager
              (c) Charles Petzold, 1990
  ----------------------------------------------------------------*/

#include <windows.h>

long FAR PASCAL WndProc (HWND, WORD, WORD, LONG) ;

#define ID_TIMER    1

int PASCAL WinMain (HANDLE hInstance, HANDLE hPrevInstance,
                    LPSTR lpszCmdLine, int nCmdShow)
     {
     char     szAppName [] = "Fader" ;
     HWND     hwnd ;
     MSG      msg ;
     WNDCLASS wndclass ;

     if (!hPrevInstance) 
          {
          wndclass.style         = CS_HREDRAW | CS_VREDRAW ;
          wndclass.lpfnWndProc   = WndProc ;
          wndclass.cbClsExtra    = 0 ;
          wndclass.cbWndExtra    = 0 ;
          wndclass.hInstance     = hInstance ;
          wndclass.hIcon         = NULL ;
          wndclass.hCursor       = LoadCursor (NULL, IDC_ARROW) ;
          wndclass.hbrBackground = GetStockObject (WHITE_BRUSH) ;
          wndclass.lpszMenuName  = szAppName ;
          wndclass.lpszClassName = szAppName ;

          RegisterClass (&wndclass) ;
          }

     hwnd = CreateWindow (szAppName, "Fader: Palette Animation",
                          WS_OVERLAPPEDWINDOW,
                          CW_USEDEFAULT, CW_USEDEFAULT,
                          CW_USEDEFAULT, CW_USEDEFAULT,
                          NULL, NULL, hInstance, NULL) ;

     ShowWindow (hwnd, nCmdShow) ;
     UpdateWindow (hwnd) ;

     while (GetMessage (&msg, NULL, 0, 0))
          {
          TranslateMessage (&msg) ;
          DispatchMessage (&msg) ;
          }
     return msg.wParam ;
     }

long FAR PASCAL WndProc (HWND hwnd, WORD message, WORD wParam, LONG lParam)
     {
     static BOOL       bFadeIn = TRUE ;
     static char       szText [] = "Fade In And Out " ;
     static HPALETTE   hPal ;
     static LOGPALETTE lp ;
     static short      cxClient, cyClient ;
     BOOL              bPalSupport ;
     HDC               hdc ;
     int               x, y, iWidth, iHeight ;
     long              lExtent ;
     PAINTSTRUCT       ps ;

     switch (message)
          {
          case WM_CREATE:

                    // Check for palette support

               hdc = GetDC (hwnd) ;

               bPalSupport = (RC_PALETTE & GetDeviceCaps (hdc, RASTERCAPS)) &&
                             (GetDeviceCaps (hdc, SIZEPALETTE) >
                              GetDeviceCaps (hdc, NUMRESERVED)) ;

               ReleaseDC (hwnd, hdc) ;

               if (!bPalSupport)
                    {
                    MessageBox (hwnd, "Program requires palette support",
                                "Fader", MB_ICONEXCLAMATION | MB_OK) ;

                    DestroyWindow (hwnd) ;
                    return 0 ;
                    }

                    // Initialize the fields of the LOGPALETTE structure

               lp.palVersion             = 0x300 ;
               lp.palNumEntries          = 1 ;
               lp.palPalEntry[0].peRed   = 255 ;
               lp.palPalEntry[0].peGreen = 255 ;
               lp.palPalEntry[0].peBlue  = 255 ;
               lp.palPalEntry[0].peFlags = PC_RESERVED ;

                    // Create the logical palette

               hPal = CreatePalette (&lp) ;

                    // Set the timer

               SetTimer (hwnd, ID_TIMER, 50, NULL) ;
               return 0 ;

          case WM_SIZE:
               cxClient = LOWORD (lParam) ;
               cyClient = HIWORD (lParam) ;
               return 0 ;

          case WM_TIMER:

                    // Define new color value

               if (bFadeIn)
                    {
                    lp.palPalEntry[0].peRed   -= 4 ;
                    lp.palPalEntry[0].peGreen -= 4 ;

                    if (lp.palPalEntry[0].peRed == 3)
                         bFadeIn = FALSE ;
                    }
               else
                    {
                    lp.palPalEntry[0].peRed   += 4 ;
                    lp.palPalEntry[0].peGreen += 4 ;

                    if (lp.palPalEntry[0].peRed == 255)
                         bFadeIn = TRUE ;
                    }

                    // Animate the palette

               AnimatePalette (hPal, 0, 1, lp.palPalEntry) ;
               return 0 ;

          case WM_PAINT:
               hdc = BeginPaint (hwnd, &ps) ;

               SelectPalette (hdc, hPal, FALSE) ;
               RealizePalette (hdc) ;

               SetTextColor (hdc, PALETTEINDEX (0)) ;

               lExtent = GetTextExtent (hdc, szText, sizeof (szText) - 1) ;
               iWidth  = LOWORD (lExtent) ;
               iHeight = HIWORD (lExtent) ;

               for (x = 0 ; x < cxClient ; x += iWidth)
               for (y = 0 ; y < cyClient ; y += iHeight)
                    {
                    TextOut (hdc, x, y, szText, sizeof (szText) - 1) ;
                    }

               EndPaint (hwnd, &ps) ;
               return 0 ;

          case WM_QUERYNEWPALETTE:
               hdc = GetDC (hwnd) ;

               SelectPalette (hdc, hPal, FALSE) ;

               if (RealizePalette (hdc) > 0)
                    {
                    ReleaseDC (hwnd, hdc) ;
                    InvalidateRect (hwnd, NULL, FALSE) ;
                    return TRUE ;
                    }
               else
                    {
                    ReleaseDC (hwnd, hdc) ;
                    return FALSE ;
                    }
               break ;

          case WM_PALETTECHANGED:
               if (wParam != hwnd)
                    {
                    hdc = GetDC (hwnd) ;

                    SelectPalette (hdc, hPal, FALSE) ;

                    if (RealizePalette (hdc) > 0)
                         {
                         InvalidateRect (hwnd, NULL, FALSE) ;
                         }

                    ReleaseDC (hwnd, hdc) ;
                    }
               return 0 ;

          case WM_DESTROY :

                    // Kill the timer

               KillTimer (hwnd, ID_TIMER) ;

                    // Delete the logical palette

               DeleteObject (hPal) ;
               PostQuitMessage (0) ;
               return 0 ;
          }
     return DefWindowProc (hwnd, message, wParam, lParam) ;
     }
